
# #!/usr/bin/env python3
#
"""Teneto is a module with tools for analyzing network patterns in time."""
#
__author__ = "William Hedley Thompson (wiheto)"
__version__ = "0.3.0"
#
import teneto.utils as utils
import teneto.networkmeasures as networkmeasures
import teneto.plot as plot
import teneto.generatenetwork as generatenetwork
import teneto.derive as derive
import teneto.misc as misc
import teneto.trajectory as trajectory
import teneto.communitydetection as communitydetection
from teneto.classes import TenetoBIDS
#del misc.teneto
#del communitydetection.static.modularity_based_clustering

#del measures
